/*
*******************************************************************************
*  Copyright (c) 1996 Martin Poole                            *
*******************************************************************************
**
** WARNING !!  WARNING !! WARNING !! WARNING !! WARNING !! WARNING !!
**
** Any changes to be made to this file should first be checked with
** mplib1 source control for library integrity.
**
** mplib1 source control can be reached at  mplib1@quatermass.co.uk
** 
*
*  System         :
*  Subsystem      :
*  Module         :
*  $Source: /home/cvs/cvsroot/onelan/onelan/src/mplib1/libsrc/gdbm_util.c,v $
*  $Author: mpoole $
*  $Date: 2002/10/07 09:37:39 $
*  $Revision: 1.2 $
*  Purpose        :
*
*******************************************************************************
*
* $Log: gdbm_util.c,v $
* Revision 1.2  2002/10/07 09:37:39  mpoole
* Initial checkin of mplib1-3.1.0
*
* Revision 1.1  2002/10/07 09:36:56  mpoole
* Initial checkin of mplib1-3.1.0
*
*
*******************************************************************************
*/

#ident "$Header: /home/cvs/cvsroot/onelan/onelan/src/mplib1/libsrc/gdbm_util.c,v 1.2 2002/10/07 09:37:39 mpoole Exp $"

/* ------------------------------------------------------------------
    Include files
   ------------------------------------------------------------------ */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <sys/ipc.h>
#include <sys/shm.h>

#include <gdbm.h>

#include <mplib1/dl_list.h>
#include <mplib1/fprintfile.h>
#include <mplib1/cfg_file.h>

#include <mplib1/gdbm_util.h>


/* ------------------------------------------------------------------
    defines
   ------------------------------------------------------------------ */

static char util_debug_str[]="GDBM_UTIL_DEBUG";
static int gdbm_util_debug=0;

/* ------------------------------------------------------------------
    Code starts here
   ------------------------------------------------------------------ */

static void
gdbm_fail( const char *errstr )
{
    fprintfile( stderr,"GDBM Error: %s\n", errstr );
    return;
}

int
gdbm_db_close( struct gdbm_help *gptr )
{
    gdbm_util_debug=get_config_flag(util_debug_str);
    if (gdbm_util_debug)
	fprintfile(stderr,"gdbm_db_close: Close (%s)\n", gptr->name );
    if ( *gptr->file_ptr )
    {
	gdbm_close( *gptr->file_ptr );
	if (gdbm_util_debug)
	    fprintfile(stderr,"gdbm_db_close: Closed\n" );
	*gptr->file_ptr = NULL;
    }
    else if (gdbm_util_debug)
	fprintfile(stderr,"gdbm_db_close: not open\n" );
    return(1);
}

static GDBM_FILE
gdbm_open_this( const char *name, int blk_sz, int o_flags,
		int cache_sz, void (*fatal_func)(const char *) )
{
    GDBM_FILE rv=NULL;
 
    if (gdbm_util_debug)
	fprintfile(stderr,"gdbm_open_this: %s %d %x\n", name, blk_sz, o_flags );
    rv = gdbm_open( (char *)name, blk_sz, o_flags, 0600, fatal_func );
    if (rv && cache_sz)
    {
	gdbm_setopt( rv, GDBM_CACHESIZE, &cache_sz, sizeof(int) );
    }
    return(rv);
}

static int
gdbm_open_one( const char *cfg_name, const char *def_name,
		const char *block_size, const char *non_buf,
		const char *cache_size, int force_new,
		void (*fatal_func)(const char *), GDBM_FILE *p_rv )
{
    GDBM_FILE rv=NULL;
    int o_flags;
    int block_sz=0,cache_sz=0;
    char *db_name;
 
    /* Do this one */
    db_name = strdup( eval_config_default( cfg_name, def_name ));
    o_flags= ((force_new)?GDBM_NEWDB : GDBM_WRCREAT);
    if (non_buf && get_config_flag(non_buf))
	o_flags |= GDBM_FAST;
    if (block_size)
	block_sz = get_config_int( block_size );
    if (cache_size)
	cache_sz = get_config_int( cache_size );
    if (fatal_func==NULL)
	fatal_func = gdbm_fail;
    rv = gdbm_open_this( db_name, block_sz, o_flags, cache_sz, fatal_func );
    if (rv==NULL)
    {
	fprintfile(stderr,"Unable to open database file (%s)\n", db_name );
    }
    free(db_name);
    *p_rv = rv;
    return((rv==NULL)?0:1);
}

int
gdbm_db_open( struct gdbm_help *gptr, int force_new )
{
    int rv=0;

    gdbm_util_debug=get_config_flag(util_debug_str);
    if (gptr->name)
    {
	if (   gptr->conditional==NULL
	    || !get_config_flag(gptr->conditional)
	   )
	{
	    rv = gdbm_open_one( gptr->name, gptr->def_name,
				gptr->blocksize, gptr->fast,
				gptr->cachesize, force_new,
				gptr->fatal_func, gptr->file_ptr );
	    if (gdbm_util_debug)
		fprintfile(stderr,"gdbm_db_open: open - rv = %d\n", rv );
	}
	else
	{
	    rv = 1;
	    if (gdbm_util_debug)
		fprintfile(stderr,"gdbm_db_open: open - conditional\n" );
	}
    }else
    if (gdbm_util_debug)
	fprintfile(stderr,"gdbm_db_open: open - no name\n" );
    return(rv);
}

int
gdbm_db_sync_set( struct gdbm_help *gptr, int sync_mode )
{
    int sv;
    sv = (sync_mode) ? 1:0;
    if (*gptr->file_ptr)
	gdbm_setopt( *gptr->file_ptr, GDBM_FASTMODE, &sv, sizeof(sv) );
    return(1);
}

char *
gdbm_key_gen( char *buf, const char *format, ... )
{
    va_list ap;

    if (buf==NULL)
	buf=malloc(255);
    if (buf)
    {
	va_start( ap, format );
	vsprintf( buf, format, ap );
	va_end( ap );
    }
    return(buf);
}

int
store_some_record( GDBM_FILE file,
		    const void *ptr,
		    size_t psize,
		    const char *format, ... )
{
    datum skey,sdata;
    char some_key[255];
    va_list ap;
    int rv;

    gdbm_util_debug=get_config_flag(util_debug_str);
    va_start( ap, format );
    vsprintf( some_key, format, ap );
    va_end( ap );
    if (gdbm_util_debug)
	fprintfile(stderr,"store_some_record: %s\n", some_key );
    skey.dptr = some_key;
    skey.dsize = strlen(some_key) + 1;
    sdata.dptr = (char *)ptr;
    sdata.dsize = psize;
    rv = gdbm_store( file, skey, sdata, GDBM_REPLACE );
    if (gdbm_util_debug)
	fprintfile(stderr,"store_some_record: rv=%p\n", rv );
    return( rv );
}

void *
fetch_some_record( GDBM_FILE file, const char *format, ... )
{
    datum skey,sdata;
    char some_key[255];
    va_list ap;

    gdbm_util_debug=get_config_flag(util_debug_str);
    va_start( ap, format );
    vsprintf( some_key, format, ap );
    va_end( ap );
    if (gdbm_util_debug)
	fprintfile(stderr,"fetch_some_record: %s\n", some_key );
    skey.dptr = some_key;
    skey.dsize = strlen(some_key) + 1;
    sdata = gdbm_fetch( file, skey );
    if (gdbm_util_debug)
	fprintfile(stderr,"fetch_some_record: rv=%p\n", sdata.dptr );
/*    DO_AREP( sdata.dptr, sdata.dsize );*/
    return( (void *)(sdata.dptr) );
}

int
delete_some_record( GDBM_FILE file, const char *format, ... )
{
    datum skey;
    int rv;
    char some_key[255];
    va_list ap;

    gdbm_util_debug=get_config_flag(util_debug_str);
    va_start( ap, format );
    vsprintf( some_key, format, ap );
    va_end( ap );
    if (gdbm_util_debug)
	fprintfile(stderr,"delete_some_record: %s\n", some_key );
    skey.dptr = some_key;
    skey.dsize = strlen(some_key) + 1;
    rv = gdbm_delete( file, skey );
    if (gdbm_util_debug)
	fprintfile(stderr,"delete_some_record: rv=%d\n", rv );
    return( rv );
}


/* -- End of File -- */
